@@ -19,11 +19,11 @@
 #define LIBPLACEBO_CONFIG_H_
 
 // Increased any time the library changes in a fundamental/major way.
-#define PL_MAJOR_VER 4
+#define PL_MAJOR_VER 5
 
 // Increased any time the API changes. (Note: Does not reset when PL_MAJOR_VER
 // is increased)
-#define PL_API_VER 208
+#define PL_API_VER 264
 
 // Increased any time a fix is made to a given API version.
 #define PL_FIX_VER (pl_fix_ver())
@@ -31,36 +31,54 @@
 // Friendly name (`git describe`) for the overall version of the library
 #define PL_VERSION (pl_version())
 
-int pl_fix_ver(void);
-const char *pl_version(void);
-
 // Feature tests. These aren't described in further detail, but may be useful
 // for programmers wanting to programmatically check for feature support
 // in their compiled libplacebo versions.
-#define PL_HAVE_LCMS 1
-#define PL_HAVE_GLSLANG 1
-#define PL_HAVE_SHADERC 1
-#define PL_HAVE_VULKAN 1
-#define PL_HAVE_OPENGL 1
 #undef PL_HAVE_D3D11
+#define PL_HAVE_DOVI 1
+#define PL_HAVE_GL_PROC_ADDR 1
+#undef PL_HAVE_GLSLANG
+#define PL_HAVE_LCMS 1
+#undef PL_HAVE_LIBDOVI
+#define PL_HAVE_OPENGL 1
+#define PL_HAVE_SHADERC 1
 #define PL_HAVE_VK_PROC_ADDR 1
+#define PL_HAVE_VULKAN 1
+
 
 // Extra compiler-specific stuff
-#if defined(_MSC_VER)
-#define PL_DEPRECATED
-#else
-#define PL_DEPRECATED __attribute__((deprecated))
+#ifndef PL_DEPRECATED
+# if defined(_MSC_VER)
+# define PL_DEPRECATED
+# else
+# define PL_DEPRECATED __attribute__((deprecated))
+# endif
+#endif
+
+#ifndef __has_feature
+#define __has_feature(x) 0
+#endif
+
+#ifndef PL_DEPRECATED_ENUMERATOR
+# if (defined(__GNUC__) && (__GNUC__ >= 6)) || __has_feature(enumerator_attributes)
+# define PL_DEPRECATED_ENUMERATOR PL_DEPRECATED
+# else
+# define PL_DEPRECATED_ENUMERATOR
+# endif
 #endif
 
 // C++ compatibility
-#ifdef __cplusplus
-# define PL_STRUCT(name) struct name##_t
-# define PL_API_BEGIN extern "C" {
-# define PL_API_END }
-#else
-# define PL_STRUCT(name) struct name
-# define PL_API_BEGIN
-# define PL_API_END
+#ifndef PL_API_BEGIN
+# ifdef __cplusplus
+#  define PL_API_BEGIN extern "C" {
+#  define PL_API_END }
+# else
+#  define PL_API_BEGIN
+#  define PL_API_END
+# endif
+#endif
+
+#ifndef __cplusplus
 // Disable this warning because libplacebo's params macros override fields
 # pragma GCC diagnostic ignored "-Woverride-init"
 #endif
@@ -69,4 +87,14 @@ const char *pl_version(void);
 #define PL_TOSTRING_INNER(x) #x
 #define PL_TOSTRING(x) PL_TOSTRING_INNER(x)
 
-#endif // LIBPLACEBO_CONTEXT_H_
+// Deprecated macro for back-compatibility
+#define PL_STRUCT(name) struct name##_t
+
+PL_API_BEGIN
+
+int pl_fix_ver(void);
+const char *pl_version(void);
+
+PL_API_END
+
+#endif // LIBPLACEBO_CONFIG_H_
