@@ -22,21 +22,23 @@
 #include <libplacebo/colorspace.h>
 #include <libplacebo/gpu.h>
 
+PL_API_BEGIN
+
 // This abstraction represents a low-level interface to visible surfaces
 // exposed by a graphics API (and accompanying GPU instance), allowing users to
 // directly present frames to the screen (or window, typically). This is a
 // sister API to gpu.h and follows the same convention w.r.t undefined behavior.
-
-struct pl_swapchain {
-    struct pl_context *ctx;   // the pl_context this swapchain was allocated from
-    struct pl_sw_fns *impl;   // the underlying implementation
-    const struct pl_gpu *gpu; // the GPU instance this swapchain belongs to
-};
+//
+// Thread-safety: Safe
+typedef const struct pl_swapchain_t {
+    pl_log log;
+    pl_gpu gpu;
+} *pl_swapchain;
 
 // Destroys this swapchain. May be used at any time, and may block until the
 // completion of all outstanding rendering commands. The swapchain and any
 // resources retrieved from it must not be used afterwards.
-void pl_swapchain_destroy(const struct pl_swapchain **sw);
+void pl_swapchain_destroy(pl_swapchain *sw);
 
 // Returns the approximate current swapchain latency in vsyncs, or 0 if
 // unknown. A latency of 1 means that `submit_frame` followed by `swap_buffers`
@@ -44,7 +46,7 @@ void pl_swapchain_destroy(const struct pl_swapchain **sw);
 // values are 2 or 3, which enable better pipelining by allowing the GPU to be
 // processing one or two frames at the same time as the user is preparing the
 // next for submission.
-int pl_swapchain_latency(const struct pl_swapchain *sw);
+int pl_swapchain_latency(pl_swapchain sw);
 
 // Update/query the swapchain size. This function performs both roles: it tries
 // setting the swapchain size to the values requested by the user, and returns
@@ -57,7 +59,26 @@ int pl_swapchain_latency(const struct pl_swapchain *sw);
 //
 // Returns false on significant errors (e.g. dead surface). This function can
 // effectively be used to probe if creating a swapchain works.
-bool pl_swapchain_resize(const struct pl_swapchain *sw, int *width, int *height);
+bool pl_swapchain_resize(pl_swapchain sw, int *width, int *height);
+
+// Backwards compatibility
+#define pl_swapchain_colors pl_color_space
+
+// Inform the swapchain about the input color space. This API deliberately
+// provides no feedback, because the swapchain can internally decide what to do
+// with this information, including ignoring it entirely, or applying it
+// asynchronously. Users must still base their rendering on the value of
+// `pl_swapchain_frame.color_space`.
+//
+// Note: Calling this function a second time completely overrides any
+// previously specified hint. So calling this on {0} or NULL resets the
+// swapchain back to its initial/preferred colorspace.
+//
+// Note: If `csp->transfer` is a HDR transfer curve but HDR metadata is left
+// unspecified, the HDR metadata defaults to `pl_hdr_metadata_hdr10`.
+// Conversely, if the HDR metadata is non-empty but `csp->transfer` is left as
+// PL_COLOR_TRC_UNKNOWN, then it instead defaults to PL_COLOR_TRC_PQ.
+void pl_swapchain_colorspace_hint(pl_swapchain sw, const struct pl_color_space *csp);
 
 // The struct used to hold the results of `pl_swapchain_start_frame`
 struct pl_swapchain_frame {
@@ -65,7 +86,7 @@ struct pl_swapchain_frame {
     // It's guaranteed that `fbo->params.renderable` and `fbo->params.blit_dst`
     // will be true, but no other guarantees are made - not even that
     // `fbo->params.format` is a real format.
-    const struct pl_tex *fbo;
+    pl_tex fbo;
 
     // If true, the user should assume that this framebuffer will be flipped
     // as a result of presenting it on-screen. If false, nothing special needs
@@ -92,8 +113,7 @@ struct pl_swapchain_frame {
 // may also be non-blocking, so users shouldn't rely on this call alone in
 // order to meter rendering speed. (Specifics depend on the underlying graphics
 // API)
-bool pl_swapchain_start_frame(const struct pl_swapchain *sw,
-                              struct pl_swapchain_frame *out_frame);
+bool pl_swapchain_start_frame(pl_swapchain sw, struct pl_swapchain_frame *out_frame);
 
 // Submits the previously started frame. Non-blocking. This must be issued in
 // lockstep with pl_swapchain_start_frame - there is no way to start multiple
@@ -103,8 +123,13 @@ bool pl_swapchain_start_frame(const struct pl_swapchain *sw,
 // platform-specific APIs for more info).
 //
 // Returns whether successful. This should normally never fail, unless the
-// GPU/surface has been lost or some other critical error has occurred.
-bool pl_swapchain_submit_frame(const struct pl_swapchain *sw);
+// GPU/surface has been lost or some other critical error has occurred. The
+// "started" frame is consumed even in the event of failure.
+//
+// Note that `start_frame` and `submit_frame` form a lock pair, i.e. trying to
+// call e.g. `pl_swapchain_resize` from another thread will block until
+// `pl_swapchain_submit_frame` is finished.
+bool pl_swapchain_submit_frame(pl_swapchain sw);
 
 // Performs a "buffer swap", or some generalization of the concept. In layman's
 // terms, this blocks until the execution of the Nth previously submitted frame
@@ -139,6 +164,8 @@ bool pl_swapchain_submit_frame(const struct pl_swapchain *sw);
 // and should not be used as an authoritative source of vsync timing
 // information without sufficient smoothing/filtering (and if so, the time that
 // `start_frame` blocked for should also be included).
-void pl_swapchain_swap_buffers(const struct pl_swapchain *sw);
+void pl_swapchain_swap_buffers(pl_swapchain sw);
+
+PL_API_END
 
 #endif // LIBPLACEBO_SWAPCHAIN_H_
